//----------------------------------------------------------------------------
//
// AATEXT - Antialiased text fonts for Allegro
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Basic aliasmap functions
//----------------------------------------------------------------------------
#include "internal.h"

/* Used for the background color */
extern int _textmode;

/* Used for scratchpad for blitting the characters */
BITMAP *_scratchpad = NULL;

/* Used for fast aliasing for filled background modes */
int _aliasmap_foreground = -1;
int _aliasmap_background = -1;
int _aliasmap_depth = -1;
int _aliasmap[8];

/*----------------------------------------------------------------------------
   Setup the global aliasmap for fast aliasing of text with
   filled backgrounds.
*/
void _setup_aliasmap (int colour, int depth)
   {
   int cr, cg, cb, tr, tg, tb;
   int(*colmaker)(int,int,int);

   int darkness, index;

   switch(depth)
      {
      case(15):
         cr = getr15(colour);
         cg = getg15(colour);
         cb = getb15(colour);

         tr = getr15(_textmode);
         tg = getg15(_textmode);
         tb = getb15(_textmode);

         colmaker = makecol15;
         break;
      case(16):
         cr = getr16(colour);
         cg = getg16(colour);
         cb = getb16(colour);

         tr = getr16(_textmode);
         tg = getg16(_textmode);
         tb = getb16(_textmode);

         colmaker = makecol16;
         break;
      case(24):
         cr = getr24(colour);
         cg = getg24(colour);
         cb = getb24(colour);

         tr = getr24(_textmode);
         tg = getg24(_textmode);
         tb = getb24(_textmode);

         colmaker = makecol24;
         break;
      case(32):
         cr = getr32(colour);
         cg = getg32(colour);
         cb = getb32(colour);

         tr = getr32(_textmode);
         tg = getg32(_textmode);
         tb = getb32(_textmode);

         colmaker = makecol32;
         break;
      default:
         cr = getr8(colour);
         cg = getg8(colour);
         cb = getb8(colour);

         tr = getr8(_textmode);
         tg = getg8(_textmode);
         tb = getb8(_textmode);

         colmaker = makecol8;
         break;
      }

   /* Build the color map */
   darkness = 37;
   for(index=1;index<7;index++)
      {
      _aliasmap[index] = colmaker((cr*darkness+(255-darkness)*tr)>>8,
                                  (cg*darkness+(255-darkness)*tg)>>8,
                                  (cb*darkness+(255-darkness)*tb)>>8);
      darkness += 37;
      }
   _aliasmap[0] = _textmode;
   _aliasmap[7] = colour;

   /* Confirm that the aliasmap has been built */
   _aliasmap_foreground = colour;
   _aliasmap_background = _textmode;
   _aliasmap_depth = depth;
   }

/*----------------------------------------------------------------------------
   Make sure the aliasmap has the correct forground and background colors
   and also prepare the scratchpad
*/
void _prepare_aliasmap (int colour, int depth)
   {
   /* Possible setup the quick color alias map */
   if((colour!=_aliasmap_foreground)||
      (_textmode!=_aliasmap_background)||
      (_aliasmap_depth!=depth))
      {
      _setup_aliasmap(colour,depth);
      }

   /* Possible setup for internal scratchpad bitmap */
   if((_scratchpad==NULL)||
      (bitmap_color_depth(_scratchpad)!=depth))
      {
      if(_scratchpad!=NULL) destroy_bitmap(_scratchpad);
      _scratchpad = create_bitmap_ex(depth,_SCRATCHPAD_WIDTH,_SCRATCHPAD_HEIGHT);
      }
   }


