//----------------------------------------------------------------------------
//
// AATEXT - Antialiased text fonts for Allegro
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Basic text drawing functions
//----------------------------------------------------------------------------
#include <stdio.h>
#include <stdarg.h>
#include "allegttf.h"
#include "internal.h"

/* Used for the background color */
extern int _textmode;

/*----------------------------------------------------------------------------
   Actual textout routine
*/
void aatextout (struct BITMAP *bmp, struct FONT *font, char *string, int x, int y, int colour)
   {
   int i;
   struct BITMAP *b;

   /* Make sure that we haven't been called with a fixed width font by accident */
   /* Also make sure that the bitmap isn't so big as to overflow the internal scratchpad */
   if((font->mono)||(text_height(font)>_SCRATCHPAD_HEIGHT))
      {
      textout(bmp,font,string,x,y,colour);
      return;
      }

   /* Blit each character */
   while(*string)
      {
      /* Find the characters bitmap */
      i = (int)(unsigned char)*string - ' ';

      /* Render the character */
      _antialiased_render(bmp,font->glyphs,i,x,y,colour);

      b = ((BITMAP**)font->glyphs)[i];
      x += b->w;
      string++;
      }
   }

/*----------------------------------------------------------------------------
   Centered textout routine
*/
void aatextout_center(struct BITMAP *bmp, struct FONT *font, char *string, int x, int y, int colour)
   {
   aatextout(bmp, font, string, x - (text_length(font, string) / 2), y, colour);
   }

/*----------------------------------------------------------------------------
   Internal character rendering function
*/
void _antialiased_render (struct BITMAP *bmp, void* glyphs, int ch, int x, int y, int colour)
   {
   BITMAP* b = ((BITMAP**)glyphs)[ch];
   
   switch(bitmap_color_depth(bmp))
      {
      case(8):
         if(_textmode>=0)
            _antialiased_8bit_filled_render(bmp,b,x,y,colour);
         else
            _antialiased_8bit_empty_render(bmp,b,x,y,colour);
         break;
         
#ifdef ALLEGRO_COLOR16
      case(15):
         if(_textmode>=0)
            _antialiased_15bit_filled_render(bmp,b,x,y,colour);
         else
            _antialiased_15bit_empty_render(bmp,b,x,y,colour);
         break;
      case(16):
         if(_textmode>=0)
            _antialiased_16bit_filled_render(bmp,b,x,y,colour);
         else
            _antialiased_16bit_empty_render(bmp,b,x,y,colour);
         break;
#endif //ALLEGRO_COLOR16

#ifdef ALLEGRO_COLOR24
      case(24):
         if(_textmode>=0)
            _antialiased_24bit_filled_render(bmp,b,x,y,colour);
         else
            _antialiased_24bit_empty_render(bmp,b,x,y,colour);
         break;
#endif //ALLEGRO_COLOR24

#ifdef ALLEGRO_COLOR32
      case(32):
         if(_textmode>=0)
            _antialiased_32bit_filled_render(bmp,b,x,y,colour);
         else
            _antialiased_32bit_empty_render(bmp,b,x,y,colour);
         break;
#endif //ALLEGRO_COLOR24
      }
   }

//----------------------------------------------------------------------------
// these printf-like routines come almost directly from Allegro
//----------------------------------------------------------------------------
/* aatextprintf:
 *  Formatted text output, using a printf() style format string.
 */
void aatextprintf(struct BITMAP *bmp, struct FONT *f, int x, int y, int color, char *format, ...)
{
   char buf[256];

   va_list ap;
   va_start(ap, format);
   vsprintf(buf, format, ap);
   va_end(ap);

   aatextout(bmp, f, buf, x, y, color);
}

/* aatextprintf_center:
 *  Like aatextprintf(), but uses the x coordinate as the centre rather than
 *  the left of the string.
 */
void aatextprintf_center(struct BITMAP *bmp, struct FONT *f, int x, int y, int color, char *format, ...)
{
   char buf[256];

   va_list ap;
   va_start(ap, format);
   vsprintf(buf, format, ap);
   va_end(ap);

   aatextout_center(bmp, f, buf, x, y, color);
}


