//----------------------------------------------------------------------------
//
// AATEXT - Antialiased text fonts for Allegro
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Basic text drawing functions
//----------------------------------------------------------------------------
#include <sys/farptr.h>
#include "internal.h"

#ifdef ALLEGRO_COLOR16

/* Used for fast blending for truecolor aliasing */
unsigned long _blender_trans16(unsigned long x, unsigned long y, unsigned long n);

/*----------------------------------------------------------------------------
   Very fast 6 level antialiased text blitting for true color modes
   and filled backgrounds.  I don't think it's possible to get
   it much faster than this...
*/
void _antialiased_16bit_filled_render (struct BITMAP *bmp, struct BITMAP *b, int x, int y, int colour)
   {
   /* Setup the internal scratchpad and aliasmap to the correct color depth */
   _prepare_aliasmap(colour,16);

   /* do the common 15 or 16 bit character drawing */
   _antialiased_15_or_16bit_filled_render(bmp,b,x,y);
   }

/*----------------------------------------------------------------------------
   Pretty fast antialiased text blitting for 16 bit modes and
   empty backgrounds. It fully supports 256 aliasing levels
*/
void _antialiased_16bit_empty_render (struct BITMAP *bmp, struct BITMAP *b, int x, int y, int colour)
   {
   int xx, i, yy, xxend;
   unsigned char *input;

   /* Set the selector for the far poke in the inner loop */
   _farsetsel(bmp->seg);

   /* Plot the aliased pixels */
   for(i = 0; i < b->h; i++)
      {
      input = b->line[i];

      /* Find the line number and check if it's clipped */
      yy = y+i;

      /* Clipping under means we are done */
      if(yy>=bmp->cb)
        return;
      
      if(yy>=bmp->ct)
         {
         /* Adjust for any LHS clipping */
         int pix = 0;
         if(x<bmp->cl)
            {
            pix = bmp->cl - x;
            input += pix;
            }

         /* Calculate where the RHS is */
         xxend = x + b->w;

         /* Adjust the RHS for clipping */
         if(xxend>=bmp->cr)
            xxend = bmp->cr;

         xxend *=2;

         for(xx = (x + pix)*2; xx < xxend; xx+=2)
            {
            /* Blend the color */
            unsigned int blendlevel = *input++;

            if (blendlevel)
               {
               /* Get the bacgground and blend it */
               unsigned int back = _farnspeekw(bmp_read_line(bmp,yy)+xx);
               back = _blender_trans16(back,colour,255-blendlevel);

               /* Plot it to the screen */
               _farnspokew(bmp_write_line(bmp,yy)+xx, back);
               }
            }
         }
      }
   }
   
#endif //ALLEGRO_COLOR16

