#ifndef _WAVEGUI_H_
#define _WAVEGUI_H_

#include <allegro.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "wavenet.h"
#include "guidata.h"

extern BITMAP *buffer;
extern FONT *font12, *font8, *fontg, *fontd;

/*
Class for handling buttons.
*/
class GUI_BUTTON
{
 private:
  int x1, y1, x2, y2;
  char name[32];
  char text[255];
  int *state;
  int num;
  bool click;
  int pkey;
  int count;
 public:
  GUI_BUTTON( int _x1, int _y1, int _x2, int _y2, int *_state, int _num, char *_name, char *_text, int _pkey );
  void draw( bool enabled = true );
};

/*
Chat line class.
*/
class GUI_TEXTLINE {
 public:
  char data[80];
  int color;
  GUI_TEXTLINE( ) {
    data[0] = '\0';
    color = 0;
  }
  void set( char *d ) {
    strcpy(data, d);
    color = 100;
  }
};

/*
Chat line holding structure.
*/
class GUI_TEXTBOX {
  GUI_TEXTLINE line[20];
  int count;
 public:
  GUI_TEXTBOX( ) { count = 0; };
  void add( char *d ) {
    line[count].set( d );
    count++;
    if (count > 19) count = 0;
  }
  void render( ) {
    int j = count-1;
    for (int i=0; i<12; i++) {
      if (j < 0) j = 19;
      textout(buffer, font, line[j].data, 15, 403-(i*10), makecol(0,150+line[j].color,100));
      if (line[j].color) line[j].color -= 10;
      j--;
    }
  }
};

/*
WaveGUI class. Handles the entire lobbying process.
*/
class WAVEGUI {
 private:
  int state;
  BITMAP *layout;
  DATAFILE *guidata;
  RGB_MAP rgb_table;
  COLOR_MAP trans_table;
  GUI_BUTTON *b1, *b2, *b3, *b4, *b5, *b6, *b7, *bokay, *bconnect, *bmenu1, *bmenu2, *bmenu3;
  GUI_TEXTBOX glines;
  WAVENET *wavenet;
  void create_menu_layout();
  void create_login_layout();
  void create_failure_layout();
  void create_main_layout(int mode);
 public:
  WAVEGUI( WAVENET *driver );
  ~WAVEGUI( );
  int initialize( );
  int user_menu( bool netavail );
  int user_login( const char *name );
  int user_lobby( void (*callback)(USER *user, BITMAP *info) );
};

bool filter_keyboard( char string[], int &pos, int max );
void rrect( BITMAP *buf, int x1, int y1, int x2, int y2, int color, int indent = 0);
void gwindow( BITMAP *buf, int x1, int y1, int x2, int y2, bool hilite );
void hilite_textout( char *string, int x, int y, int &count );
int  mouse_over(int x1, int y1, int x2, int y2);
void draw_button( int x1, int y1, int x2, int y2, char *name, int hilite, bool enabled = true );
void b_textout_centre(BITMAP *bmp, FONT *f, char *str, int x, int y, int color);
void b_textout(BITMAP *bmp, FONT *f, char *str, int x, int y, int color);

#endif

/* END */
