#ifndef _WAVEIPX_H_
#define _WAVEIPX_H_

#define MY_SOCKET 0x7777
#define MAX_PACKETS_IN_BUFFER 100
#define IPX_DELAY 0
#ifndef TRUE
  #define TRUE  1
  #define FALSE 0
#endif
#define DATASIZE 250
#define SEEOWNPACKETS 1

/* from dosbuff.cc originally */
#define DOS_BUFFER_SIZE 400 /* 600 bytes */
#define DOS_BUFFER_TEST 200 /* The amount we REALLY need for the dos buffer */

typedef unsigned char BOOLEAN;
typedef unsigned char NETADD[4];
typedef unsigned char NODEADD[6];
typedef unsigned short ADDRESS[2];

typedef struct {
  NETADD          netadd        __attribute__((packed));  /* Network address */
  NODEADD         nodeadd       __attribute__((packed));  /* Node address */
  unsigned short  socket        __attribute__((packed));  /* Big endian socket number */
} IPXADDRESS;

typedef struct {
  NETADD          netadd        __attribute__((packed));  /* Network address */
  NODEADD         nodeadd       __attribute__((packed));  /* Node address */
} LOCAL_IPXADDRESS;

typedef struct {
  ADDRESS           link              __attribute__((packed)); /* Pointer to next ECB */
  unsigned long     ESR               __attribute__((packed)); /* Event service routine 00000000h if none */
  unsigned char     in_use            __attribute__((packed)); /* In use flag */
  unsigned char     complete          __attribute__((packed)); /* Completing flag */
  unsigned short    socket            __attribute__((packed)); /* Big endian socket number */
  unsigned char     IPX_work[4]       __attribute__((packed)); /* IPX work space */
  unsigned char     D_work[12]        __attribute__((packed)); /* Driver work space */
  NODEADD           immediate_address __attribute__((packed)); /* Immediate local node address */
  unsigned short    fragment_count    __attribute__((packed)); /* Fragment count */
  unsigned long     fragment_data     __attribute__((packed)); /* Pointer to data fragment */
  unsigned short    fragment_size     __attribute__((packed)); /* Size of data fragment */
} IPXECB;

typedef struct {
  unsigned short    checksum          __attribute__((packed)); /* Big endian checksum */
  unsigned short    length            __attribute__((packed)); /* Big endian length in bytes */
  unsigned char     transport_control __attribute__((packed)); /* Transport control */
  unsigned char     packet_type       __attribute__((packed)); /* Packet type */
  IPXADDRESS destination              __attribute__((packed)); /* Destination network address */
  IPXADDRESS source                   __attribute__((packed)); /* Source network address */
} IPXHEADER;

typedef unsigned char DATA[DATASIZE];

typedef struct {
  IPXECB    ecb         __attribute__((packed));
  IPXHEADER ipx_header  __attribute__((packed));
  DATA      data        __attribute__((packed));
} INTERNAL_PACKET;

typedef struct {
 DATA data;
 NODEADD node;
} PACKET;

typedef struct {
 PACKET packets[MAX_PACKETS_IN_BUFFER];
 unsigned short buffer_start;
 unsigned short buffer_pos;
} PACKET_BUFFER;

extern LOCAL_IPXADDRESS local_address;

int Compare_Nodes( NODEADD node_one, NODEADD node_two );
unsigned short Endian_Swap( unsigned short old_short );
int IPX_Open_Socket( unsigned char longetivity, unsigned short *socket_number );
void IPX_Close_Socket( unsigned short *socket_number );
void Get_Local_Address();
void IPX_Send_Packet( IPXECB *ecb );
int IPX_Listen_For_Packet( IPXECB *ecb );
void Im_Idle();
void Init_Send_Packet( IPXECB *ecb, IPXHEADER *ipx_header, unsigned short size, unsigned short sock );
void Init_Recieve_Packet( IPXECB *ecb, IPXHEADER *ipx_header, unsigned short size, unsigned short sock );
int Init_IPX();

/* from dosbuff.cc originally */
void init_dosbuff( void );
void Copy_Into_Dos_Buffer( void *block, short length );
void Copy_From_Dos_Buffer( void *block, short length );
void Copy_Into_Dos_Buffer_Two( void *block, short length ) ;
void Copy_From_Dos_Buffer_Two( void *block, short length );
void Copy_Into_Send_Buffer( void *block, short length );
void Copy_From_Send_Buffer( void *block, short length );
void Copy_Into_Recieve_Buffer( void *block, short length );
void Copy_From_Recieve_Buffer( void *block, short length );
void Copy_Into_Ecb_Send_Buffer( void *block, short length );
void Copy_From_Ecb_Send_Buffer( void *block, short length );
void Copy_Into_Ecb_Recieve_Buffer( void *block, short length );
void Copy_From_Ecb_Recieve_Buffer( void *block, short length );
void Test_Dos_Buffers( void );
unsigned long Make_Far_Pointer( unsigned short segment, unsigned short offset );

/*
 The following are basically all the commands that will be used, all
 the above are for internal purposes only.
*/

/* Initializes buffers, IPX, and opens a socket. Returns 1 if successful. */
int init_waveipx();
/* Shuts down IPX drivers and closes the socket. */
void close_waveipx();
/* Returns 1 if a packet is waiting for retrieval in the buffer. */
int packet_ready();
/* Tries to get a packet from the buffer, returns 1 if one was retrieved. */
int pop_packet( PACKET *packet );
/* Send a packet. Ignore the return, its always 1. */
int push_packet( DATA data, int length, NODEADD dest_node );

#endif


